<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 5/9/17
 * Time: 8:56 AM
 */

namespace qfq;

require_once(__DIR__ . '/../exceptions/CodeException.php');
require_once(__DIR__ . '/../helper/Logger.php');
require_once(__DIR__ . '/Database.php');

/*
 * Read the extension version number.
 * Read the QFQ database version number: stored in the comment field of table 'Form'. Format:  Version=x.y.z
 * If versions different:
 *   * Read the update array 'DatabaseUpdateData.php'.
 *   * Play all changes from the update array after 'old' upto 'new'.
 *   * Save new QFQ database version in the comment field of table 'Form'
 *
 * In a new QFQ installation, the comment field of table 'Form' is empty. On the first call of QFQ, the version string
 * will be set. Also the 'formEditor.sql' will be played initially.
 *
 */

/**
 * Class DatabaseUpdate
 * @package qfq
 */
class DatabaseUpdate {

    /**
     * @var Database instantiated class
     */
    protected $db = null;

    /**
     * @param Database $db
     */
    public function __construct(Database $db) {
        $this->db = $db;
    }

    /**
     * @return mixed
     * @throws CodeException
     */
    private function getExtensionVersion() {
        $path = __DIR__ . '/../../../ext_emconf.php';
        $_EXTKEY = EXT_KEY;
        $EM_CONF = null;
        if (@file_exists($path)) {
            include $path;

            if (isset($EM_CONF[$_EXTKEY]['version'])) {
                return $EM_CONF[$_EXTKEY]['version'];
            }
        }

        throw new CodeException('Failed to read extension version', ERROR_QFQ_VERSION);
    }

    /**
     * Try to read the QFQ version number from 'comment' in table 'Form'.
     * In a very special situation , there might be table 'form' and 'Form' in the same Database. This should be handled
     *  in the way that the latest version number is the active one.
     *
     * @return bool|string  false if there is no table 'Form' or if there is no comment set in table 'Form'.
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     */
    private function getDatabaseVersion() {

        $arr = $this->db->sql("SHOW TABLE STATUS WHERE Name='Form'", ROW_REGULAR);

        $found = '';
        //
        foreach ($arr as $row) {
            if (isset($row['Comment'])) {
                $arr = explode('=', $row['Comment']);
                if (count($arr) == 2 && $arr[0] == 'Version' && $arr[1] != '' AND (version_compare($arr[1], $found) == 1)) {
                    $found = $arr[1];
                }
            } else {
                continue;
            }
        }

        return $found === '' ? false : $found;
    }

    /**
     * @param $version
     *
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     */
    private function setDatabaseVersion($version) {

        $this->db->sql("ALTER TABLE `Form` COMMENT = 'Version=$version'");

    }

    /**
     *
     * @param string $dbUpdate SYSTEM_DB_UPDATE_ON | SYSTEM_DB_UPDATE_OFF | SYSTEM_DB_UPDATE_AUTO
     *
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     */
    public function checkNupdate($dbUpdate) {

        if ($dbUpdate === SYSTEM_DB_UPDATE_NEVER) {
            return;
        }

        $new = $this->getExtensionVersion();
        $old = $this->getDatabaseVersion();

        if ($dbUpdate === SYSTEM_DB_UPDATE_ALWAYS || ($dbUpdate === SYSTEM_DB_UPDATE_AUTO && $new != $old)) {
            $this->dbUpdateStatements($old, $new);
            $this->db->playSqlFile(__DIR__ . '/../../sql/formEditor.sql');

            $qfqLog = $this->db->getQfqLogFile();
            Logger::logMessage(date('Y.m.d H:i:s ') . ": Updated from QFQ version '$old' to '$new'", $qfqLog);

            // Finally write the latest version number.
            $this->setDatabaseVersion($new);
        }

        if ($old === false) {
            // A complete new installation get's some extra tables
            $this->db->playSqlFile(__DIR__ . '/../../sql/customTable.sql');
        }
    }

    /**
     * @param $path
     *
     * @return array
     * @throws CodeException
     */
    private function readUpdateData($path) {

        if (!@file_exists($path)) {
            throw new CodeException("File '$path'' not found", ERROR_IO_OPEN);
        }

        $UPDATE_ARRAY = null;

        include $path;

        return $UPDATE_ARRAY;

    }

    /**
     * Play all update statement with version number are '>' than $old and '<=' to $new.
     *
     * @param $old
     * @param $new
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     */
    private function dbUpdateStatements($old, $new) {
        if ($new == '' || $old === false) {
            return;
        }
        $updateArray = $this->readUpdateData(__DIR__ . '/DatabaseUpdateData.php');

        $apply = false;
        foreach ($updateArray as $key => $sqlStatements) {

            // Search starting point to apply updates. Do not apply updates for $key>$new
            $rc1 = version_compare($key, $old);
            $rc2 = version_compare($key, $new);
            if ($rc1 == 1 && $rc2 != 1) {
                $apply = true;
            }

            if ($apply) {
                // Play Statements
                foreach ($sqlStatements as $sql) {
                    $this->db->sql($sql, ROW_REGULAR, array(), "Apply updates to QFQ database. Installed version: $old. New QFQ version: $new");
                }
                // Remember already applied updates - in case something breaks and the update has to be repeated.
                $this->setDatabaseVersion($new);
            }
        }
    }
}